<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

defined('MOODLE_INTERNAL') || die();

/**
 * Provides support for the conversion of moodle1 backup to the moodle2 format
 * Based off of a template @ http://docs.moodle.org/dev/Backup_1.9_conversion_for_developers
 *
 * @package    mod
 * @subpackage pagemenu
 * @copyright  2011 Valery Fremaux <valery.fremaux@club-internet.fr>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

/**
 * Pagemenu conversion handler
 */
class moodle1_mod_pagemenu_handler extends moodle1_mod_handler {

    /** @var moodle1_file_manager */
    protected $fileman = null;

    /** @var int cmid */
    protected $moduleid = null;

    /**
     * Declare the paths in moodle.xml we are able to convert
     *
     * The method returns list of {@link convert_path} instances.
     * For each path returned, the corresponding conversion method must be
     * defined.
     *
     * Note that the path /MOODLE_BACKUP/COURSE/MODULES/MOD/PAGEMENU does not
     * actually exist in the file. The last element with the module name was
     * appended by the moodle1_converter class.
     *
     * @return array of {@link convert_path} instances
     */
    public function get_paths() {
        return array(
            new convert_path(
                'pagemenu', '/MOODLE_BACKUP/COURSE/MODULES/MOD/PAGEMENU',
                array(
                    'newfields' => array(
                        'intro' => '',
                        'introformat' => FORMAT_MOODLE,
                    ),
                )
            ),
            new convert_path(
                'pagemenu_links', '/MOODLE_BACKUP/COURSE/MODULES/MOD/PAGEMENU/LINKS',
                array(
                )
            ),
            new convert_path(
                'pagemenu_link', '/MOODLE_BACKUP/COURSE/MODULES/MOD/PAGEMENU/LINKS/LINK',
                array(
                )
            ),
            new convert_path(
                'pagemenu_link_data', '/MOODLE_BACKUP/COURSE/MODULES/MOD/PAGEMENU/LINKS/LINK/DATA',
                array(
                )
            ),
            new convert_path(
                'pagemenu_link_datum', '/MOODLE_BACKUP/COURSE/MODULES/MOD/PAGEMENU/LINKS/LINK/DATA/DATUM',
                array(
                )
            ),
       );
    }

    /**
     * This is executed every time we have one /MOODLE_BACKUP/COURSE/MODULES/MOD/PAGEMENU
     * data available
     */
    public function process_pagemenu($data) {

        // Get the course module id and context id.
        $instanceid = $data['id'];
        $cminfo     = $this->get_cminfo($instanceid);
        $moduleid   = $cminfo['id'];
        $contextid  = $this->converter->get_contextid(CONTEXT_MODULE, $moduleid);

        // Get a fresh new file manager for this instance.
        $this->fileman = $this->converter->get_file_manager($contextid, 'mod_pagemenu');

        // Convert course files embedded into the intro.
        $data['intro'] = '';
        $data['introformat'] = FORMAT_MOODLE;

        // Write inforef.xml.
        $this->open_xml_writer("activities/pagemenu_{$moduleid}/inforef.xml");
        $this->xmlwriter->begin_tag('inforef');
        $this->xmlwriter->begin_tag('fileref');
        foreach ($this->fileman->get_fileids() as $fileid) {
            $this->write_xml('file', array('id' => $fileid));
        }
        $this->xmlwriter->end_tag('fileref');
        $this->xmlwriter->end_tag('inforef');
        $this->close_xml_writer();

        // Write pagemenu.xml.
        $this->open_xml_writer("activities/pagemenu_{$moduleid}/pagemenu.xml");
        $this->xmlwriter->begin_tag('activity', array('id' => $instanceid, 'moduleid' => $moduleid,
            'modulename' => 'pagemenu', 'contextid' => $contextid));

        $this->xmlwriter->begin_tag('pagemenu', array('id' => $instanceid));

        foreach ($data as $field => $value) {
            if ($field <> 'id') {
                $this->xmlwriter->full_tag($field, $value);
            }
        }

        return $data;
    }

    /**
     * This is executed when we reach the closing </MOD> tag of our 'pagemenu' path
     */
    public function on_pagemenu_end() {

        // Finish writing pagemenu.xml.
        $this->xmlwriter->end_tag('pagemenu');
        $this->xmlwriter->end_tag('activity');
        $this->close_xml_writer();

        // Write inforef.xml.
        $this->open_xml_writer("activities/pagemenu_{$this->moduleid}/inforef.xml");
        $this->xmlwriter->begin_tag('inforef');
        $this->xmlwriter->begin_tag('fileref');
        $this->xmlwriter->end_tag('fileref');
        $this->xmlwriter->end_tag('inforef');
        $this->close_xml_writer();
    }

    /* Links */
    public function on_pagemenu_links_start() {
        $this->xmlwriter->begin_tag('links');
    }

    public function on_pagemenu_links_end() {
        $this->xmlwriter->end_tag('links');
    }

    /* Link */
    public function on_pagemenu_link_start() {
        $this->xmlwriter->begin_tag('link');
    }

    public function on_pagemenu_link_end() {
        $this->xmlwriter->end_tag('link');
    }

    // Write link content.
    public function process_pagemenu_link($data) {
        global $currentlinkid;

        $currentlinkid = $data['id'];
        $this->write_xml('link', $data);
    }

    /* Link datas */
    /* ELEMENT ITEM */
    public function on_pagemenu_link_data_start() {
        $this->xmlwriter->begin_tag('data');
    }

    public function on_pagemenu_link_data_end() {
        $this->xmlwriter->end_tag('data');
    }

    // Process datum in one single write.
    public function process_pagemenu_link_datum($datum) {
        global $currentlinkid;

        // This regenerates a missing linkid expected in new datum records.
        $datum['linkid'] = $currentlinkid;
        $this->write_xml('datum', $datum);
    }

}
